#  Copyright (c) 2020.  by Dheeraj Gupta.
#  This is part of json_flatten project.
#  Project: json_flatten
#  Module:
#  File: flatten.py
#  Author: Dheeraj Gupta
#  Description : tbd

# Global import
from itertools import chain, starmap


# Main Code


def flatten(dictionary):
    """
        Flatten a nested dictionary structure

    >>> flatten({'abc': {'a': 24, 'b': {'b1': {'size': 3, 'out': 'Nope'}, 'size': True}}, 'xyz': {'x': {'word': 8}, 'y': -1, 'z': 26}, 'pqr': {'pq': [0, None, 2.0, 3.0], 'r': None}})
    {'abc[a]': 24, 'abc[b][b1][size]': 3, 'abc[b][b1][out]': 'Nope', 'abc[b][size]': True, 'xyz[x][word]': 8, 'xyz[y]': -1, 'xyz[z]': 26, 'pqr[pq][0]': 0, 'pqr[pq][1]': None, 'pqr[pq][2]': 2.0, 'pqr[pq][3]': 3.0, 'pqr[r]': None}

    """

    def unpack(parent_key, parent_value):
        """Unpack one level of nesting in a dictionary"""
        try:
            items = parent_value.items()
        except AttributeError:
            # parent_value was not a dict, no need to flatten
            yield (parent_key, parent_value)
        else:
            for key, value in items:
                if type(value) == list:
                    for k, v in enumerate(value):
                        yield parent_key + '[' + key + ']' + '['+str(k)+']', v
                else:
                    yield parent_key + '['+key+']', value
    while True:
        # Keep unpacking the dictionary until all value's are not dictionary's
        dictionary = dict(chain.from_iterable(starmap(unpack, dictionary.items())))
        if not any(isinstance(value, dict) for value in dictionary.values()):
            break
    return dictionary


#  Copyright (c) 2020.  by Dheeraj Gupta.
#  This is part of json_flatten project.
#  Project: json_flatten
#  Module:
#  File: __init__.py
#  Author: Dheeraj Gupta
#  Description : tbd

# Global import
from .flatten import flatten
from .unflatten import unflatten

# Main Code


__all__ = ['flatten', 'unflatten']




#  Copyright (c) 2020.  by Dheeraj Gupta.
#  This is part of json_flatten project.
#  Project: json_flatten
#  Module:
#  File: unflatten.py
#  Author: Dheeraj Gupta
#  Description : tbd

# Global import

# Main Code


json = {'draw': '3', 'columns[0][data]': '0', 'columns[0][name]': '', 'columns[0][searchable]': 'true', 'columns[0][orderable]': 'false', 'columns[0][search][value]': '', 'columns[0][search][regex]': 'false', 'order[0][column]': '1', 'order[0][dir]': 'asc', 'start': '0', 'length': '13', 'search[value]': 'jenkins', 'search[regex]': 'false', 'searchPanes[group][0]': 'Group 1', 'searchPanes[platform][0]': 'Window', '_': '1597573303858'}

def unflatten(json_dict):
    """
        Convert the JSON Flat dictionary into nested one.

    >>> unflatten({'draw': '3', 'columns[0][data]': '0', 'columns[0][name]': '', 'columns[0][searchable]': 'true', 'columns[0][orderable]': 'false', 'columns[0][search][value]': '', 'columns[0][search][regex]': 'false', 'order[0][column]': '1', 'order[0][dir]': 'asc', 'start': '0', 'length': '13', 'search[value]': 'jenkins', 'search[regex]': 'false', 'searchPanes[group][0]': 'Group 1', 'searchPanes[platform][0]': 'Window', '_': '1597573303858'})
    {'draw': '3', 'columns': [{'data': '0', 'name': '', 'searchable': 'true', 'orderable': 'false', 'search': {'value': '', 'regex': 'false'}}], 'order': [{'column': '1', 'dir': 'asc'}], 'start': '0', 'length': '13', 'search': {'value': 'jenkins', 'regex': 'false'}, 'searchPanes': {'group': ['Group 1'], 'platform': ['Window']}, '_': '1597573303858'}

    """
    import re
    dictionary = {}
    for key, value in json_dict.items():
        temp_dict = dictionary
        tokens = re.findall(r'\w+', key)
        for count, (index, next_token) in enumerate(zip(tokens, tokens[1:] + [value]), 1):
            value = next_token if count == len(tokens) else [] if next_token.isdigit() else {}
            if isinstance(temp_dict, list):
                index = int(index)
                while index >= len(temp_dict):
                    temp_dict.append(value)
            elif index not in temp_dict:
                temp_dict[index] = value
            temp_dict = temp_dict[index]
    return dictionary


from setuptools import setup, find_packages

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='json_python_flatten',
    packages=find_packages(include=['json_python_flatten']),
    version='0.1.3',
    description='Flatten JSON objects',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    author='Dheeraj Gupta',
    author_email='drg0072@gmail.com',
    url='https://github.com/drg0072/json_flatten',
    keywords=['json', 'flatten'],
    classifiers=[],
    entry_points={},
    install_requires=[]
)
